@@ -25,7 +25,7 @@ import math
 import pickle
 from decimal import Decimal
 from textwrap import dedent
-from typing import Optional, List, Tuple
+from typing import Optional
 from xml.etree import ElementTree
 
 try:
@@ -34,8 +34,8 @@ except ImportError:
     lxml_etree = None
 
 from elementpath import datatypes, XPath1Parser, XPathContext, MissingContextError, \
-    AttributeNode, NamespaceNode, TextNode, CommentNode, ProcessingInstructionNode, \
-    ElementNode, select, XPathFunction
+    NamespaceNode, TextNode, CommentNode, ProcessingInstructionNode, select, XPathFunction
+from elementpath.xpath_nodes import TextAttributeNode, EtreeElementNode
 from elementpath.namespaces import XSD_NAMESPACE, XPATH_FUNCTIONS_NAMESPACE, \
     XPATH_MATH_FUNCTIONS_NAMESPACE
 from elementpath.sequence_types import is_sequence_type
@@ -384,7 +384,7 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
     def test_node_types(self):
         element = self.etree.Element('schema')
         context = XPathContext(element)
-        attribute = AttributeNode('id', '0212349350')
+        attribute = TextAttributeNode('id', '0212349350')
         namespace = NamespaceNode('xs', 'http://www.w3.org/2001/XMLSchema')
         comment = CommentNode(self.etree.Comment('nothing important'))
         pi = ProcessingInstructionNode(self.etree.ProcessingInstruction('action'))
@@ -449,11 +449,6 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         context = XPathContext(root)
         self.check_value('./B/@xml:id[id("bar")]', expected=[], context=context)
 
-        context.item = None
-        self.check_value('id("none")', expected=[], context=context)
-        self.check_value('id("foo")', expected=[context.root[0]], context=context)
-        self.check_value('id("bar")', expected=[context.root[2]], context=context)
-
         context.item = CommentNode(self.etree.Comment('a comment'))
         self.check_value('id("foo")', expected=[], context=context)
 
@@ -1240,19 +1235,19 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_value('/A', [context.root], context=context)
 
     def test_path_step_operator_with_duplicates(self):
-        root = self.etree.XML('<A>10<B a="2">11</B>10<B a="2"/>10<B>11</B></A>')
+        root = self.etree.XML('<A>10<B a="2">11</B>12<B a="2"/>13<B>14</B></A>')
         self.check_selector('/A/node()', root,
-                            ['10', root[0], '10', root[1], '10', root[2]])
+                            ['10', root[0], '12', root[1], '13', root[2]])
         self.check_selector('/A/node() | /A/node()', root,
-                            ['10', root[0], '10', root[1], '10', root[2]])
+                            ['10', root[0], '12', root[1], '13', root[2]])
         self.check_selector('/A/node() | /A/B/text()', root,
-                            ['10', root[0], '11', '10', root[1], '10', root[2], '11'])
+                            ['10', root[0], '11', '12', root[1], '13', root[2], '14'])
 
-        root = self.etree.XML('<A a="2"><B1 a="2"/><B1 a="2"/><B1 a="3"/><B2 a="2"/></A>')
-        self.check_selector('/A/B1/@a', root, ['2', '2', '3'])
-        self.check_selector('/A/B1/@a | /A/B1/@a', root, ['2', '2', '3'])
-        self.check_selector('/A/B1/@a | /A/@a', root, ['2', '2', '2', '3'])
-        self.check_selector('/A/B1/@a | /A/B2/@a', root, ['2', '2', '3', '2'])
+        root = self.etree.XML('<A a="1"><B1 a="2"/><B1 a="3"/><B1 a="4"/><B2 a="5"/></A>')
+        self.check_selector('/A/B1/@a', root, ['2', '3', '4'])
+        self.check_selector('/A/B1/@a | /A/B1/@a', root, ['2', '3', '4'])
+        self.check_selector('/A/B1/@a | /A/@a', root, ['1', '2', '3', '4'])
+        self.check_selector('/A/B1/@a | /A/B2/@a', root, ['2', '3', '4', '5'])
 
     def test_context_item_expression(self):
         root = self.etree.XML('<A><B1><C/></B1><B2/><B3><C1/><C2/></B3></A>')
@@ -1333,16 +1328,16 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
 
         root = self.etree.XML("<A><B><C><D><E/></D></C></B></A>")
         context = XPathContext(root)
-        expected = list(e for e in context.root.iter() if isinstance(e, ElementNode))
+        expected = list(e for e in context.root.iter() if isinstance(e, EtreeElementNode))
         self.check_value('//*', expected=expected, context=context)
 
         context = XPathContext(root, item=root[0][0])
-        expected = list(e for e in context.root.iter() if isinstance(e, ElementNode))
+        expected = list(e for e in context.root.iter() if isinstance(e, EtreeElementNode))
         self.check_value('//*', expected=expected, context=context)
 
         root = self.etree.XML("<A><A><A><A><A/></A></A></A></A>")
         context = XPathContext(root)
-        expected = list(e for e in context.root.iter() if isinstance(e, ElementNode))
+        expected = list(e for e in context.root.iter() if isinstance(e, EtreeElementNode))
         self.check_value('//A', expected=expected, context=context)
 
     def test_double_slash_shortcut_pr16(self):
@@ -1567,10 +1562,7 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
             self.check_selector('/*:foo', root, [root], namespaces={'': 'ns'})  # foo --> {ns}foo
 
         root = self.etree.XML('<foo xmlns="ns">bar</foo>')
-        if self.parser.version == '1.0' or self.etree is not lxml_etree:
-            self.check_selector('/foo', root, [])
-        else:
-            self.check_selector('/foo', root, [root])
+        self.check_selector('/foo', root, [])
 
         if self.parser.version == '1.0':
             self.check_selector('/foo', root, [], namespaces={'': 'ns'})
@@ -1654,6 +1646,19 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
 
         self.check_selector("//target[name=//var/name]", root, expected=[root[0]])
 
+    def test_order_with_descendants__issue_079(self):
+        xml_data = ("<span><span><span>Achille Compagnoni </span></span> and <span>"
+                    "<span><span>Lino Lacedelli</span></span></span></span>")
+        root = self.etree.fromstring(xml_data)
+
+        xpath_expr = '//descendant-or-self::text()'
+        chunks = select(root, xpath_expr)
+        self.assertListEqual(chunks, ['Achille Compagnoni ', ' and ', 'Lino Lacedelli'])
+
+        xpath_expr = '//text()'
+        chunks = select(root, xpath_expr)
+        self.assertListEqual(chunks, ['Achille Compagnoni ', ' and ', 'Lino Lacedelli'])
+
     def test_get_function(self):
         func = self.parser.get_function('fn:true', 0)
         self.assertEqual(str(func), "'fn:true' function")
@@ -1732,7 +1737,7 @@ class LxmlXPath1ParserTest(XPath1ParserTest):
 
     def test_namespace_axis(self):
         root = self.etree.XML('<A xmlns:tst="http://xpath.test/ns"><tst:B1/></A>')
-        namespaces: List[Tuple[Optional[str], str]] = []
+        namespaces: list[tuple[Optional[str], str]] = []
         namespaces.extend(self.parser.DEFAULT_NAMESPACES.items())
         namespaces += [('tst', 'http://xpath.test/ns')]
 
