@@ -44,7 +44,8 @@ else:
 
 from elementpath import XPath2Parser, XPathContext, ElementPathError, \
     MissingContextError, select, Selector, datatypes, get_node_tree, \
-    AttributeNode, NamespaceNode, TextNode
+    NamespaceNode, TextNode
+from elementpath.xpath_nodes import TextAttributeNode
 from elementpath.namespaces import XSI_NAMESPACE, XML_NAMESPACE, XML_ID
 from elementpath.datatypes import DateTime10, DateTime, Date10, Date, Time, \
     Timezone, DayTimeDuration, YearMonthDuration, QName, UntypedAtomic
@@ -1119,10 +1120,10 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
             root = self.etree.XML('<root/>')
             self.wrong_value("data(/root)", 'FOTY0012',
                              'argument node', 'does not have a typed value',
-                             context=XPathContext(root))
+                             context=XPathContext(root, schema=self.parser.schema))
             self.wrong_value("data(.)", 'FOTY0012',
                              'argument node', 'does not have a typed value',
-                             context=XPathContext(root))
+                             context=XPathContext(root, schema=self.parser.schema))
         finally:
             self.parser.schema = None
 
@@ -1150,9 +1151,9 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         root = doc.getroot()
 
         self.check_selector("id('ID21256')", doc, [root])
-        self.check_selector("id('E21256')", doc, [root[0]])
+        self.check_selector("id('E21256')", doc, [])
         self.check_selector('element-with-id("ID21256")', doc, [root])
-        self.check_selector('element-with-id("E21256")', doc, [root])
+        self.check_selector('element-with-id("E21256")', doc, [])
 
         with self.assertRaises(MissingContextError) as err:
             self.check_value("id('ID21256')")
@@ -1173,7 +1174,7 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
 
         # Id on root element
         root = self.etree.XML("<empnr>E21256</empnr>")
-        self.check_selector("id('E21256')", root, [root])
+        self.check_selector("id('E21256')", root, [])
         self.check_selector('element-with-id("E21256")', root, [])
 
     @unittest.skipIf(xmlschema is None, "xmlschema library is not installed ...")
@@ -1203,10 +1204,8 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
             </xs:schema>"""))
 
         self.assertTrue(schema.is_valid(root))
-        with self.schema_bound_parser(schema.xpath_proxy):
-            context = XPathContext(doc)
-            self.check_select("id('ID21256')", [context.root.getroot()], context)
-            # self.check_select("id('E21256')", [root[0]], context)
+        context = XPathContext(doc, schema=schema.xpath_proxy)
+        self.check_select("id('ID21256')", [context.root.getroot()], context)
 
         # Test with matching value of type xs:string
         schema = xmlschema.XMLSchema(dedent("""\
@@ -1225,9 +1224,8 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
             </xs:schema>"""))
 
         self.assertTrue(schema.is_valid(root))
-        with self.schema_bound_parser(schema.xpath_proxy):
-            context = XPathContext(doc)
-            self.check_select("id('E21256')", [], context)
+        context = XPathContext(doc, schema=self.parser.schema)
+        self.check_select("id('E21256')", [], context)
 
     @unittest.skipIf(xmlschema is None, "xmlschema library is not installed ...")
     def test_node_set_id_function_with_wrong_schema(self):
@@ -1245,10 +1243,9 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
             </xs:schema>"""))
 
         self.assertFalse(schema.is_valid(root))
-        with self.schema_bound_parser(schema.xpath_proxy):
-            context = XPathContext(doc)
-            self.check_select("id('ID21256')", [context.root.getroot()], context)
-            self.check_select("id('E21256')", [], context)
+        context = XPathContext(doc, schema=schema.xpath_proxy)
+        self.check_select("id('ID21256')", [context.root.getroot()], context)
+        self.check_select("id('E21256')", [], context)
 
         schema = xmlschema.XMLSchema(dedent("""\
             <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
@@ -1256,10 +1253,9 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
             </xs:schema>"""))
 
         self.assertFalse(schema.is_valid(root))
-        with self.schema_bound_parser(schema.xpath_proxy):
-            context = XPathContext(doc)
-            self.check_select("id('ID21256')", [context.root.getroot()], context)
-            self.check_select("id('E21256')", [], context)
+        context = XPathContext(doc, schema=schema.xpath_proxy)
+        self.check_select("id('ID21256')", [context.root.getroot()], context)
+        self.check_select("id('E21256')", [], context)
 
     def test_node_set_idref_function(self):
         doc = self.etree.parse(io.StringIO("""
@@ -1290,13 +1286,18 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
 
         context = XPathContext(doc, item=root)
         context.variables = {
-            'x': AttributeNode(XML_ID, 'ID21256', parent=context.root[0])
+            'x': TextAttributeNode(XML_ID, 'ID21256', parent=context.root[0])
         }
         self.check_value("idref('ID21256', $x)", [], context=context)
 
         context = XPathContext(root, variables={'x': None})
-        context.item = None
-        self.check_value("idref('ID21256', $x)", [], context=context)
+        self.wrong_type("idref('ID21256', $x)", 'XPTY0004', context=context)
+
+        context = XPathContext(root, variables={'x': []})
+        self.wrong_type("idref('ID21256', $x)", 'XPTY0004', context=context)
+
+        context = XPathContext(root)
+        self.wrong_type("idref('ID21256', ())", 'XPTY0004', context=context)
 
     def test_deep_equal_function(self):
         root = self.etree.XML("""
@@ -1365,8 +1366,8 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         self.check_value('deep-equal($a, $b)', False, context=context)
 
         context = XPathContext(root)
-        context.variables = {'a': [AttributeNode('a', '10')],
-                             'b': [AttributeNode('b', '10')]}
+        context.variables = {'a': [TextAttributeNode('a', '10')],
+                             'b': [TextAttributeNode('b', '10')]}
         self.check_value('deep-equal($a, $a)', True, context=context)
         self.check_value('deep-equal($a, $b)', False, context=context)
 
@@ -1655,7 +1656,7 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
     def test_error_function(self):
         with self.assertRaises(ElementPathError) as err:
             self.check_value('fn:error()')
-        self.assertEqual(str(err.exception), '[err:FOER0000] Unidentified error')
+        self.assertIn('[err:FOER0000] Unidentified error', str(err.exception))
 
         with self.assertRaises(ElementPathError) as err:
             self.check_value('fn:error("err:XPST0001")')
@@ -1665,14 +1666,14 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
             self.check_value(
                 "fn:error(fn:QName('http://www.w3.org/2005/xqt-errors', 'err:XPST0001'))"
             )
-        self.assertEqual(str(err.exception), '[err:XPST0001] Parser not bound to a schema')
+        self.assertIn('[err:XPST0001] Parser not bound to a schema', str(err.exception))
 
         with self.assertRaises(ElementPathError) as err:
             self.check_value(
                 "fn:error(fn:QName('http://www.w3.org/2005/xqt-errors', 'err:XPST0001'), "
                 "'Missing schema')"
             )
-        self.assertEqual(str(err.exception), '[err:XPST0001] Missing schema')
+        self.assertIn('[err:XPST0001] Missing schema', str(err.exception))
 
     def test_trace_function(self):
         self.check_value('trace((), "trace message")', [])
