@@ -25,23 +25,12 @@ import locale
 import os
 from decimal import Decimal
 from textwrap import dedent
+import xml.etree.ElementTree as ET
 
-try:
-    import lxml.etree as lxml_etree
-except ImportError:
-    lxml_etree = None
-
-try:
-    import xmlschema
-except ImportError:
-    xmlschema = None
-else:
-    xmlschema.XMLSchema.meta_schema.build()
-
-from elementpath import XPath2Parser, XPathContext, MissingContextError, \
-    ElementNode, select, iter_select
-from elementpath.datatypes import xsd10_atomic_types, xsd11_atomic_types, DateTime, \
-    Date, Time, Timezone, DayTimeDuration, YearMonthDuration, UntypedAtomic, QName
+from elementpath import XPath2Parser, XPathContext, XPathSchemaContext, \
+    MissingContextError, ElementNode, select, iter_select, get_node_tree
+from elementpath.datatypes import xsd_atomic_types, DateTime, Date, Date10, \
+    Time, Timezone, DayTimeDuration, YearMonthDuration, UntypedAtomic, QName
 from elementpath.namespaces import XPATH_FUNCTIONS_NAMESPACE
 from elementpath.collations import get_locale_category
 from elementpath.sequence_types import is_instance
@@ -52,6 +41,22 @@ try:
 except ImportError:
     import test_xpath1_parser
 
+try:
+    import lxml.etree as lxml_etree
+    import lxml.html as lxml_html
+except ImportError:
+    lxml_etree = None
+    lxml_html = None
+
+try:
+    import xmlschema
+    from xmlschema.xpath import XMLSchemaProxy
+except ImportError:
+    xmlschema = None
+    XMLSchemaProxy = None
+else:
+    xmlschema.XMLSchema.meta_schema.build()
+
 
 def get_sequence_type(value, xsd_version='1.0'):
     """
@@ -76,18 +81,15 @@ def get_sequence_type(value, xsd_version='1.0'):
         if QName.is_valid(value) and ':' in str(value):
             return 'xs:QName'
 
-        if xsd_version == '1.0':
-            atomic_types = xsd10_atomic_types
-        else:
-            atomic_types = xsd11_atomic_types
-            if atomic_types['dateTimeStamp'].is_valid(value):
+        if xsd_version == '1.1':
+            if xsd_atomic_types['1.1']['dateTimeStamp'].is_valid(value):
                 return 'xs:dateTimeStamp'
 
         for type_name in ['string', 'boolean', 'decimal', 'float', 'double',
                           'date', 'dateTime', 'gDay', 'gMonth', 'gMonthDay', 'anyURI',
                           'gYear', 'gYearMonth', 'time', 'duration', 'dayTimeDuration',
                           'yearMonthDuration', 'base64Binary', 'hexBinary']:
-            if atomic_types[type_name].is_valid(value):
+            if xsd_atomic_types[xsd_version][type_name].is_valid(value):
                 return 'xs:%s' % type_name
 
     raise ValueError("Inconsistent sequence type for {!r}".format(value))
@@ -133,7 +135,9 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
 
         token = self.parser.parse('$var1')
         self.assertEqual(token.source, '$var1')
-        self.assertEqual(repr(token), f'_DollarSignOperator({self.parser})')
+        self.assertEqual(
+            repr(token), f"<{token.__class__.__name__} object at {hex(id(token))}>"
+        )
         self.assertEqual(str(token), '$var1 variable reference')
 
         context = XPathContext(root=root, variables={'var1': root[0]})
@@ -538,20 +542,30 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
 
             with self.schema_bound_parser(schema.elements['root'].xpath_proxy):
                 root = self.etree.XML('<root>11</root>')
-                self.check_value('. le 10', False, context=XPathContext(root))
-                self.check_value('. le 20', True, context=XPathContext(root))
+                context = XPathContext(root, schema=self.parser.schema)
+                self.check_value('. le 10', False, context)
+                self.check_value('. le 20', True, context)
 
                 root = self.etree.XML('<root>eleven</root>')
-                self.wrong_type('. le 10', 'XPDY0050', context=XPathContext(root))
+                context = XPathContext(root, schema=self.parser.schema)
+                self.wrong_value('. le 10', 'FORG0001', context=context)
 
                 root = self.etree.XML('<value>12</value>')
+                context = XPathContext(root, schema=self.parser.schema)
                 with self.assertRaises(TypeError) as err:
-                    self.check_value('. le "11"', context=XPathContext(root))
+                    self.check_value('. le "11"', context)
                 self.assertIn('XPTY0004', str(err.exception))  # Static schema context error
 
-                with self.assertRaises(TypeError) as err:
-                    self.check_value('. le 10', context=XPathContext(root))
-                self.assertIn('XPTY0004', str(err.exception))  # Dynamic context error
+                self.check_value('. le 10', False, context=context)
+
+                # Schema information persists on parser (will be removed in v5.0)
+                context = XPathContext(root, schema=self.parser.schema)
+                self.check_value('. le 10', False, context=context)
+
+            context = XPathContext(root)
+            with self.assertRaises(TypeError) as err:
+                self.check_value('. le 10', context=context)
+            self.assertIn('XPTY0004', str(err.exception))  # Dynamic context error
 
             schema = xmlschema.XMLSchema("""
                 <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
@@ -570,6 +584,19 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
         self.check_selector("a[3] = (1 to 10, 30)", root, True)
         self.check_selector("a[4] = (1 to 10, 30)", root, False)
 
+    @unittest.skipIf(xmlschema is None, "xmlschema library is not installed!")
+    def test_namespace_axis_on_schema_context(self):
+        schema = xmlschema.XMLSchema(dedent("""\n
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                <xs:simpleType name="floatType">
+                    <xs:restriction base="xs:double"/>
+                </xs:simpleType>
+            </xs:schema>"""))
+
+        context = XPathSchemaContext(schema)
+        token = self.parser.parse('/namespace::*')
+        self.assertListEqual(token.evaluate(context), [])
+
     def test_unknown_axis(self):
         self.wrong_syntax('unknown::node()', 'XPST0003')
         self.wrong_syntax('A/unknown::node()', 'XPST0003')
@@ -773,10 +800,12 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
     def test_attribute_accessor(self):
         root = self.etree.XML('<A a="10" b="20">text<B/>tail<B/></A>')
         context = XPathContext(root)
-        self.check_select("attribute()", {'10', '20'}, context)
-        self.check_select("attribute(*)", {'10', '20'}, context)
-        self.check_select("attribute(a)", ['10'], context)
-        self.check_select("attribute(a, xs:int)", ['10'], context)
+        a = context.root.attributes[0]
+        b = context.root.attributes[1]
+        self.check_select("attribute()", [a, b], context)
+        self.check_select("attribute(*)", [a, b], context)
+        self.check_select("attribute(a)", [a], context)
+        self.check_select("attribute(a, xs:int)", [a], context)
 
         if xmlschema is not None:
             schema = xmlschema.XMLSchema("""
@@ -788,9 +817,13 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
                   </xs:complexType>
                 </xs:schema>""")
 
-            with self.schema_bound_parser(schema.elements['A'].xpath_proxy):
-                self.check_select("attribute(a, xs:int)", [10], context)
-                self.check_select("attribute(*, xs:int)", {10, 20}, context)
+            schema_proxy = schema.elements['A'].xpath_proxy
+            with self.schema_bound_parser(schema_proxy):
+                context = XPathContext(root, schema=schema_proxy)
+                a = context.root.attributes[0]
+                b = context.root.attributes[1]
+                self.check_select("attribute(a, xs:int)", [a], context)
+                self.check_select("attribute(*, xs:int)", [a, b], context)
                 self.check_select("attribute(a, xs:string)", [], context)
                 self.check_select("attribute(*, xs:string)", [], context)
 
@@ -800,7 +833,7 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
 
         context = XPathContext(root=element)
         self.check_select("self::node()", [context.root], context)
-        self.check_select("self::attribute()", ['0212349350'], context)
+        self.check_select("self::attribute()", [context.root.attributes[0]], context)
 
         context.item = 7
         self.check_select("node()", [], context)
@@ -946,7 +979,7 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
 
         context = XPathContext(element)
         self.check_value(". instance of element()", True, context)
-        context.item = None
+        context.item = "foo"
         self.check_value(". instance of element()", False, context)
 
         self.check_value("(5, 6) instance of xs:integer", False)
@@ -1154,12 +1187,13 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
         self.check_value('for', [context.root.getroot()], context)
 
     def test_auxiliary_tokens(self):
+        # Tokens are parsed as names, so raise at evaluation if the contest is None
         self.check_raise('as', MissingContextError)
         self.check_raise('of', MissingContextError)
 
         context = XPathContext(self.etree.XML('<root/>'))
-        self.check_raise('as', MissingContextError, context=context)
-        self.check_raise('of', MissingContextError, context=context)
+        self.check_value('as', expected=None, context=context)
+        self.check_value('of', expected=None, context=context)
 
     def test_function_namespace(self):
         function_namespace = "http://xpath.test/fn/xpath-functions"
@@ -1385,11 +1419,118 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
         self.assertEqual(parser.parse('foo(77)').evaluate(), '77')
         self.assertRaises(TypeError, parser.parse, 'foo(77.0)')
 
+    @unittest.skipIf(xmlschema is None, "xmlschema library is not installed!")
+    def test_raw_resolution_for_issue_73(self):
+
+        xsd_source = dedent("""\
+        <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+            <xs:element name="values">
+                <xs:complexType>
+                    <xs:choice maxOccurs="unbounded">
+                        <xs:element name="decimal_value" type="xs:decimal"/>
+                        <xs:element name="string_value" type="xs:string"/>
+                        <xs:element name="boolean_value" type="xs:boolean"/>
+                        <xs:element name="datetime_value" type="xs:dateTime"/>
+                        <xs:element name="date_value" type="xs:date"/>
+                    </xs:choice>
+                </xs:complexType>
+            </xs:element>
+        </xs:schema>""")
+
+        xml_source = dedent("""\
+        <values>
+            <decimal_value>3.14</decimal_value>
+            <string_value>foo</string_value>
+            <boolean_value>true</boolean_value>
+            <datetime_value>2018-01-23T12:34:56Z</datetime_value>
+            <date_value>2018-01-23</date_value>
+        </values>""")
+
+        schema = xmlschema.XMLSchema(xsd_source)
+        assert schema.is_valid(xml_source)
+
+        root = ET.fromstring(xml_source)
+
+        root_node = get_node_tree(root)
+        date_node = root_node.elements.get(root[4])
+        assert date_node.name == 'date_value'
+        assert date_node.xsd_type is None
+        assert date_node.typed_value == '2018-01-23'
+
+        schema_proxy = XMLSchemaProxy(schema)
+        parser = XPath2Parser(schema=schema_proxy)
+
+        assert date_node.xsd_type is None
+        assert date_node.typed_value == '2018-01-23'
+
+        root_token = parser.parse('fn:data(//*)')
+        assert date_node.xsd_type is None
+        assert date_node.typed_value == '2018-01-23'
+
+        context = XPathContext(root_node, schema=schema_proxy)
+        result = root_token.get_results(context)
+
+        assert date_node.typed_value == Date10(2018, 1, 23)
+        assert len(result) == 5
+        assert result[-1] == Date10(2018, 1, 23)
+
+        token = XPath2Parser().parse('fn:data(.)')
+        context = XPathContext(root_node, item=date_node)
+        result = token.get_results(context)
+
+        assert len(result) == 1
+        assert result[-1] == Date10(2018, 1, 23)
+
+    def test_proxy_token_disambiguation__issue_078(self):
+        root = self.etree.XML(dedent('''\
+            <table frame="all" rowsep="1" colsep="1" id="flowers_table">
+                <title>Flowers</title>
+                <tgroup cols="3">
+                  <colspec colname="c1" colnum="1" colwidth="1.0*"/>
+                  <colspec colname="c2" colnum="2" colwidth="1.0*"/>
+                  <colspec colname="c3" colnum="3" colwidth="1.0*"/>
+                  <thead>
+                    <row>
+                      <entry>Flower</entry>
+                      <entry>Type</entry>
+                    </row>
+                  </thead>
+                  <tbody>
+                    <row>
+                      <entry>Chrysanthemum</entry>
+                      <entry>perennial</entry>
+                      <entry>well drained</entry>
+                    </row>
+                    <row>
+                      <entry>Gardenia</entry>
+                      <entry>perennial</entry>
+                    </row>
+                    <row>
+                      <entry>Gerbera</entry>
+                      <entry>annual</entry>
+                      <entry>sandy, well-drained</entry>
+                    </row>
+                    <row>
+                      <entry>Iris</entry>
+                    </row>
+                  </tbody>
+                </tgroup>
+              </table>'''))
+
+        results = select(root, 'min(.//row/count(entry))', parser=self.parser.__class__)
+        self.assertEqual(results, 1)
+
 
 @unittest.skipIf(lxml_etree is None, "The lxml library is not installed")
 class LxmlXPath2ParserTest(XPath2ParserTest):
     etree = lxml_etree
 
+    def test_input_selector(self):
+        body = '<input/>'
+        root = self.etree.fromstring(body, parser=lxml_html.HTMLParser())
+        result = select(root, '//input')
+        self.assertEqual(len(result), 1)
+
 
 if __name__ == '__main__':
     unittest.main()
